@@ -3,12 +3,49 @@
 # License : BSD-3-Clause
 
 from functools import partial
+
 import numpy as np
 
-from ..parallel import parallel_func, check_n_jobs
-from ..io.pick import _picks_to_idx
-from ..utils import logger, verbose, _time_mask, _check_option
-from .multitaper import psd_array_multitaper
+from ..parallel import parallel_func
+from ..utils import logger, verbose, deprecated, _check_option
+
+_psd_deprecation_msg = (
+    'Function psd_{0}() is deprecated; for Raw/Epochs/Evoked instances use '
+    'spectrum = instance.compute_psd(method="{0}") instead, followed by '
+    'spectrum.get_data(return_freqs=True).')
+
+
+# adapted from SciPy
+# https://github.com/scipy/scipy/blob/f71e7fad717801c4476312fe1e23f2dfbb4c9d7f/scipy/signal/_spectral_py.py#L2019  # noqa: E501
+def _median_biases(n):
+    # Compute the biases for 0 to max(n, 1) terms included in a median calc
+    biases = np.ones(n + 1)
+    # The original SciPy code is:
+    #
+    # def _median_bias(n):
+    #     ii_2 = 2 * np.arange(1., (n - 1) // 2 + 1)
+    #     return 1 + np.sum(1. / (ii_2 + 1) - 1. / ii_2)
+    #
+    # This is a sum over (n-1)//2 terms.
+    # The ii_2 terms here for different n are:
+    #
+    # n=0: []  # 0 terms
+    # n=1: []  # 0 terms
+    # n=2: []  # 0 terms
+    # n=3: [2]  # 1 term
+    # n=4: [2]  # 1 term
+    # n=5: [2, 4]  # 2 terms
+    # n=6: [2, 4]  # 2 terms
+    # ...
+    #
+    # We can get the terms for 0 through n using a cumulative summation and
+    # indexing:
+    if n >= 3:
+        ii_2 = 2 * np.arange(1, (n - 1) // 2 + 1)
+        sums = 1 + np.cumsum(1. / (ii_2 + 1) - 1. / ii_2)
+        idx = np.arange(2, n) // 2 - 1
+        biases[3:] = sums[idx]
+    return biases
 
 
 def _decomp_aggregate_mask(epoch, func, average, freq_sl):
@@ -18,7 +55,9 @@ def _decomp_aggregate_mask(epoch, func, average, freq_sl):
     if average == 'mean':
         spect = np.nanmean(spect, axis=-1)
     elif average == 'median':
-        spect = np.nanmedian(spect, axis=-1)
+        biases = _median_biases(spect.shape[-1])
+        idx = (~np.isnan(spect)).sum(-1)
+        spect = np.nanmedian(spect, axis=-1) / biases[idx]
     return spect
 
 
@@ -54,40 +93,14 @@ def _check_nfft(n, n_fft, n_per_seg, n_overlap):
     return n_fft, n_per_seg, n_overlap
 
 
-def _check_psd_data(inst, tmin, tmax, picks, proj, reject_by_annotation=False):
-    """Check PSD data / pull arrays from inst."""
-    from ..io.base import BaseRaw
-    from ..epochs import BaseEpochs
-    from ..evoked import Evoked
-    if not isinstance(inst, (BaseEpochs, BaseRaw, Evoked)):
-        raise ValueError('epochs must be an instance of Epochs, Raw, or'
-                         'Evoked. Got type {}'.format(type(inst)))
-
-    time_mask = _time_mask(inst.times, tmin, tmax, sfreq=inst.info['sfreq'])
-    picks = _picks_to_idx(inst.info, picks, 'data', with_ref_meg=False)
-    if proj:
-        # Copy first so it's not modified
-        inst = inst.copy().apply_proj()
-
-    sfreq = inst.info['sfreq']
-    if isinstance(inst, BaseRaw):
-        start, stop = np.where(time_mask)[0][[0, -1]]
-        rba = 'NaN' if reject_by_annotation else None
-        data = inst.get_data(picks, start, stop + 1, reject_by_annotation=rba)
-    elif isinstance(inst, BaseEpochs):
-        data = inst.get_data(picks=picks)[:, :, time_mask]
-    else:  # Evoked
-        data = inst.data[picks][:, time_mask]
-
-    return data, sfreq
-
-
 @verbose
 def psd_array_welch(x, sfreq, fmin=0, fmax=np.inf, n_fft=256, n_overlap=0,
-                    n_per_seg=None, n_jobs=1, average='mean', window='hamming',
-                    verbose=None):
+                    n_per_seg=None, n_jobs=None, average='mean',
+                    window='hamming', *, verbose=None):
     """Compute power spectral density (PSD) using Welch's method.
 
+    Welch's method is described in :footcite:t:`Welch1967`.
+
     Parameters
     ----------
     x : array, shape=(..., n_times)
@@ -108,10 +121,10 @@ def psd_array_welch(x, sfreq, fmin=0, fmax=np.inf, n_fft=256, n_overlap=0,
         Length of each Welch segment (windowed with a Hamming window). Defaults
         to None, which sets n_per_seg equal to n_fft.
     %(n_jobs)s
-    %(average-psd)s
+    %(average_psd)s
 
         .. versionadded:: 0.19.0
-    %(window-psd)s
+    %(window_psd)s
 
         .. versionadded:: 0.22.0
     %(verbose)s
@@ -131,8 +144,14 @@ def psd_array_welch(x, sfreq, fmin=0, fmax=np.inf, n_fft=256, n_overlap=0,
     Notes
     -----
     .. versionadded:: 0.14.0
+
+    References
+    ----------
+    .. footbibliography::
     """
-    _check_option('average', average, (None, 'mean', 'median'))
+    _check_option('average', average, (None, False, 'mean', 'median'))
+    if average is False:
+        average = None
 
     dshape = x.shape[:-1]
     n_times = x.shape[-1]
@@ -153,8 +172,6 @@ def psd_array_welch(x, sfreq, fmin=0, fmax=np.inf, n_fft=256, n_overlap=0,
     freqs = freqs[freq_sl]
 
     # Parallelize across first N-1 dimensions
-    n_jobs = check_n_jobs(n_jobs)
-    x_splits = np.array_split(x, n_jobs)
     logger.debug(
         f'Spectogram using {n_fft}-point FFT on {n_per_seg} samples with '
         f'{n_overlap} overlap and {window} window')
@@ -163,6 +180,7 @@ def psd_array_welch(x, sfreq, fmin=0, fmax=np.inf, n_fft=256, n_overlap=0,
     parallel, my_spect_func, n_jobs = parallel_func(_spect_func, n_jobs=n_jobs)
     func = partial(spectrogram, noverlap=n_overlap, nperseg=n_per_seg,
                    nfft=n_fft, fs=sfreq, window=window)
+    x_splits = np.array_split(x, n_jobs)
     f_spect = parallel(my_spect_func(d, func=func, freq_sl=freq_sl,
                                      average=average)
                        for d in x_splits)
@@ -174,28 +192,23 @@ def psd_array_welch(x, sfreq, fmin=0, fmax=np.inf, n_fft=256, n_overlap=0,
     return psds, freqs
 
 
+@deprecated(_psd_deprecation_msg.format('welch'))
 @verbose
 def psd_welch(inst, fmin=0, fmax=np.inf, tmin=None, tmax=None, n_fft=256,
-              n_overlap=0, n_per_seg=None, picks=None, proj=False, n_jobs=1,
-              reject_by_annotation=True, average='mean', window='hamming',
+              n_overlap=0, n_per_seg=None, picks=None, proj=False, n_jobs=None,
+              reject_by_annotation=True, average='mean', window='hamming', *,
               verbose=None):
     """Compute the power spectral density (PSD) using Welch's method.
 
     Calculates periodograms for a sliding window over the time dimension, then
-    averages them together for each channel/epoch.
+    optionally averages them together for each channel/epoch.
 
     Parameters
     ----------
     inst : instance of Epochs or Raw or Evoked
         The data for PSD calculation.
-    fmin : float
-        Min frequency of interest.
-    fmax : float
-        Max frequency of interest.
-    tmin : float | None
-        Min time of interest.
-    tmax : float | None
-        Max time of interest.
+    %(fmin_fmax_psd)s
+    %(tmin_tmax_psd)s
     n_fft : int
         The length of FFT used, must be ``>= n_per_seg`` (default: 256).
         The segments will be zero-padded if ``n_fft > n_per_seg``.
@@ -208,16 +221,15 @@ def psd_welch(inst, fmin=0, fmax=np.inf, tmin=None, tmax=None, n_fft=256,
         Length of each Welch segment (windowed with a Hamming window). Defaults
         to None, which sets n_per_seg equal to n_fft.
     %(picks_good_data_noref)s
-    proj : bool
-        Apply SSP projection vectors. If inst is ndarray this is not used.
+    %(proj_psd)s
     %(n_jobs)s
     %(reject_by_annotation_raw)s
 
         .. versionadded:: 0.15.0
-    %(average-psd)s
+    %(average_psd)s
 
         .. versionadded:: 0.19.0
-    %(window-psd)s
+    %(window_psd)s
 
         .. versionadded:: 0.22.0
     %(verbose)s
@@ -236,6 +248,8 @@ def psd_welch(inst, fmin=0, fmax=np.inf, tmin=None, tmax=None, n_fft=256,
 
     See Also
     --------
+    Spectrum
+    EpochsSpectrum
     mne.io.Raw.plot_psd
     mne.Epochs.plot_psd
     psd_multitaper
@@ -245,20 +259,20 @@ def psd_welch(inst, fmin=0, fmax=np.inf, tmin=None, tmax=None, n_fft=256,
     -----
     .. versionadded:: 0.12.0
     """
-    # Prep data
-    data, sfreq = _check_psd_data(inst, tmin, tmax, picks, proj,
-                                  reject_by_annotation=reject_by_annotation)
-    return psd_array_welch(data, sfreq, fmin=fmin, fmax=fmax, n_fft=n_fft,
-                           n_overlap=n_overlap, n_per_seg=n_per_seg,
-                           average=average, n_jobs=n_jobs, window=window,
-                           verbose=verbose)
+    spectrum = inst.compute_psd(
+        'welch', fmin=fmin, fmax=fmax, tmin=tmin, tmax=tmax, picks=picks,
+        proj=proj, n_jobs=n_jobs, verbose=verbose, n_fft=n_fft,
+        n_overlap=n_overlap, n_per_seg=n_per_seg, average=average,
+        window=window)
+    return spectrum.get_data(return_freqs=True)
 
 
+@deprecated(_psd_deprecation_msg.format('multitaper'))
 @verbose
 def psd_multitaper(inst, fmin=0, fmax=np.inf, tmin=None, tmax=None,
                    bandwidth=None, adaptive=False, low_bias=True,
                    normalization='length', picks=None, proj=False,
-                   n_jobs=1, reject_by_annotation=False, verbose=None):
+                   n_jobs=None, reject_by_annotation=False, *, verbose=None):
     """Compute the power spectral density (PSD) using multitapers.
 
     Calculates spectral density for orthogonal tapers, then averages them
@@ -270,14 +284,8 @@ def psd_multitaper(inst, fmin=0, fmax=np.inf, tmin=None, tmax=None,
     ----------
     inst : instance of Epochs or Raw or Evoked
         The data for PSD calculation.
-    fmin : float
-        Min frequency of interest.
-    fmax : float
-        Max frequency of interest.
-    tmin : float | None
-        Min time of interest.
-    tmax : float | None
-        Max time of interest.
+    %(fmin_fmax_psd)s
+    %(tmin_tmax_psd)s
     bandwidth : float
         The bandwidth of the multi taper windowing function in Hz. The default
         value is a window half-bandwidth of 4.
@@ -287,13 +295,9 @@ def psd_multitaper(inst, fmin=0, fmax=np.inf, tmin=None, tmax=None,
     low_bias : bool
         Only use tapers with more than 90%% spectral concentration within
         bandwidth.
-    normalization : str
-        Either "full" or "length" (default). If "full", the PSD will
-        be normalized by the sampling rate as well as the length of
-        the signal (as in nitime).
+    %(normalization)s
     %(picks_good_data_noref)s
-    proj : bool
-        Apply SSP projection vectors. If inst is ndarray this is not used.
+    %(proj_psd)s
     %(n_jobs)s
     %(reject_by_annotation_raw)s
     %(verbose)s
@@ -309,6 +313,8 @@ def psd_multitaper(inst, fmin=0, fmax=np.inf, tmin=None, tmax=None,
 
     See Also
     --------
+    Spectrum
+    EpochsSpectrum
     mne.io.Raw.plot_psd
     mne.Epochs.plot_psd
     psd_array_multitaper
@@ -323,10 +329,9 @@ def psd_multitaper(inst, fmin=0, fmax=np.inf, tmin=None, tmax=None,
     ----------
     .. footbibliography::
     """
-    # Prep data
-    data, sfreq = _check_psd_data(inst, tmin, tmax, picks, proj,
-                                  reject_by_annotation=reject_by_annotation)
-    return psd_array_multitaper(data, sfreq, fmin=fmin, fmax=fmax,
-                                bandwidth=bandwidth, adaptive=adaptive,
-                                low_bias=low_bias, normalization=normalization,
-                                n_jobs=n_jobs, verbose=verbose)
+    spectrum = inst.compute_psd(
+        'multitaper', fmin=fmin, fmax=fmax, tmin=tmin, tmax=tmax, picks=picks,
+        proj=proj, reject_by_annotation=reject_by_annotation, n_jobs=n_jobs,
+        verbose=verbose, bandwidth=bandwidth, adaptive=adaptive,
+        low_bias=low_bias, normalization=normalization)
+    return spectrum.get_data(return_freqs=True)
