@@ -13,13 +13,13 @@ import os.path as op
 from pathlib import Path
 import re
 import sys
-import warnings
 import numbers
 
 import numpy as np
 
 from ..fixes import _median_complex, _compare_version
-from ._logging import warn, logger, verbose
+from ._logging import (warn, logger, verbose, _record_warnings,
+                       _verbose_safe_false)
 
 
 def _ensure_int(x, name='unknown', must_be='an int', *, extra=''):
@@ -273,8 +273,9 @@ def _check_preload(inst, msg):
     from ..epochs import BaseEpochs
     from ..evoked import Evoked
     from ..time_frequency import _BaseTFR
+    from ..time_frequency.spectrum import BaseSpectrum
 
-    if isinstance(inst, (_BaseTFR, Evoked)):
+    if isinstance(inst, (_BaseTFR, Evoked, BaseSpectrum)):
         pass
     else:
         name = "epochs" if isinstance(inst, BaseEpochs) else 'raw'
@@ -864,7 +865,8 @@ def _check_sphere(sphere, info=None, sphere_units='m'):
         assert info is not None
 
         if sphere == 'auto':
-            R, r0, _ = fit_sphere_to_headshape(info, verbose=False, units='m')
+            R, r0, _ = fit_sphere_to_headshape(
+                info, verbose=_verbose_safe_false(), units='m')
             sphere = tuple(r0) + (R,)
             sphere_units = 'm'
         elif sphere == 'eeglab':
@@ -975,9 +977,9 @@ def _suggest(val, options, cutoff=0.66):
         return ' Did you mean one of %r?' % (options,)
 
 
-def _check_on_missing(on_missing, name='on_missing'):
+def _check_on_missing(on_missing, name='on_missing', *, extras=()):
     _validate_type(on_missing, str, name)
-    _check_option(name, on_missing, ['raise', 'warn', 'ignore'])
+    _check_option(name, on_missing, ['raise', 'warn', 'ignore'] + list(extras))
 
 
 def _on_missing(on_missing, msg, name='on_missing', error_klass=None):
@@ -1006,23 +1008,16 @@ def _safe_input(msg, *, alt=None, use=None):
 
 def _ensure_events(events):
     err_msg = f'events should be a NumPy array of integers, got {type(events)}'
-    if check_version('numpy', '1.24'):
-        # NumPy >= 1.24 raises ValueError
+    with _record_warnings():
         try:
             events = np.asarray(events)
         except ValueError as np_err:
             if str(np_err).startswith(
                     'setting an array element with a sequence. The requested '
                     'array has an inhomogeneous shape'):
-                raise TypeError(err_msg)
+                raise TypeError(err_msg) from None
             else:
                 raise
-    else:
-        # NumPy < 1.24 gives VisibleDeprecationWarning
-        with warnings.catch_warnings(record=True):
-            warnings.simplefilter('ignore')
-            events = np.asarray(events)
-    # now that we're sure that events is an array...
     if not np.issubdtype(events.dtype, np.integer):
         raise TypeError(err_msg)
     if events.ndim != 2 or events.shape[1] != 3:
