@@ -6,8 +6,8 @@
 #
 # License: Simplified BSD
 
-import os.path as op
 from collections import namedtuple
+import os.path as op
 
 import numpy as np
 import pytest
@@ -19,6 +19,7 @@ from mne import (read_events, Epochs, pick_channels_evoked, read_cov,
 from mne.channels import read_layout
 from mne.io import read_raw_fif
 from mne.time_frequency.tfr import AverageTFR
+from mne.utils import _record_warnings
 
 from mne.viz import (plot_topo_image_epochs, _get_presser,
                      mne_analyze_colormap, plot_evoked_topo)
@@ -89,7 +90,7 @@ def test_plot_joint():
     axes = plt.subplots(nrows=3)[-1].flatten().tolist()
     evoked.plot_joint(times=[0], picks=[6, 7, 8], ts_args=dict(axes=axes[0]),
                       topomap_args={"axes": axes[1:], "time_unit": "s"})
-    with pytest.raises(ValueError, match='array of length 4'):
+    with pytest.raises(ValueError, match='of length 4'):
         evoked.plot_joint(picks=[6, 7, 8], ts_args=dict(axes=axes[0]),
                           topomap_args=dict(axes=axes[2:]))
     plt.close('all')
@@ -131,6 +132,13 @@ def test_plot_topo():
     plot_evoked_topo([evoked, evoked], merge_grads=True,
                      background_color='w')
 
+    plot_evoked_topo([evoked, evoked], merge_grads=True,
+                     background_color='w', color='blue')
+
+    with pytest.raises(ValueError, match='must be .*tuple, list, str,.*'):
+        plot_evoked_topo([evoked, evoked], merge_grads=True,
+                         color=np.array(["blue", "red"]))
+
     picked_evoked = evoked.copy().pick_channels(evoked.ch_names[:3])
     picked_evoked_eeg = evoked.copy().pick_types(meg=False, eeg=True)
     picked_evoked_eeg.pick_channels(picked_evoked_eeg.ch_names[:3])
@@ -270,6 +278,12 @@ def test_plot_tfr_topo():
     fig = tfr.plot_topo(baseline=(None, 0), mode='ratio',
                         title='Average power', vmin=0., vmax=14.)
 
+    # test complex
+    tfr.data = tfr.data * (1 + 1j)
+    plt.close('all')
+    fig = tfr.plot_topo(baseline=(None, 0), mode='ratio',
+                        title='Average power', vmin=0., vmax=14.)
+
     # test opening tfr by clicking
     num_figures_before = len(plt.get_fignums())
     # could use np.reshape(fig.axes[-1].images[0].get_extent(), (2, 2)).mean(1)
@@ -290,7 +304,7 @@ def test_plot_tfr_topo():
     # one timesample
     tfr = AverageTFR(epochs.info, data[:, :, [0]], epochs.times[[1]],
                      freqs, nave)
-    with pytest.warns(None):  # matplotlib equal left/right
+    with _record_warnings():  # matplotlib equal left/right
         tfr.plot([4], baseline=None, vmax=14., show=False, yscale='linear')
 
     # one frequency bin, log scale required: as it doesn't make sense
